package org.wfp.mpos.utils;

import android.content.Context;
import android.os.Build;
import android.telephony.TelephonyManager;
import android.util.Log;

import org.wfp.mpos.utils.sn.SNUtils;
import org.wfp.mpos.utils.sn.Constants;

import java.lang.reflect.Method;

/**
 * Created by othomas on 06/01/2018.
 */

public class FamocoIdUtils {

    public static String getFamocoId(Context context) {
        String famocoId;

        if (isFamocoDevice()) {
            if (SNUtils.isFamocoIdValid(getRoSerialNo())) {
                famocoId = getRoSerialNo();
            } else if (SNUtils.isFamocoIdValid(getGsmSerial())) {
                famocoId = getGsmSerial();
            } else {
                famocoId = getFamocoIdFromImei(context);
            }
        } else {
            famocoId = getNonFamocoSerialNumber();
        }
        return famocoId == null ? "" : famocoId.toUpperCase().replaceAll(" ","");
    }

    public static boolean isFamocoDevice() {
        return Build.MODEL.startsWith("FX");
    }

    public static String getDeviceModel() {
        return Build.MODEL;
    }

    public static String getRoSerialNo() {
        return Build.SERIAL.toUpperCase();
    }

    public static String getGsmSerial() {
        return getSystemProperty(Constants.GMS_SERIAL);
    }

    public static String getNonFamocoSerialNumber() {
        if (Build.MODEL.equals(Constants.MODEL_MOBIPRINT)) {
            return getGsmSerial();
        } else {
            return Build.SERIAL;
        }
    }

    private static String getFamocoIdFromImei(Context context) {
        TelephonyManager tm = (TelephonyManager)
                context.getSystemService(Context.TELEPHONY_SERVICE);
        String imei = tm.getDeviceId();
        return SNUtils.imeiToBarcode(imei, false);
    }

    public static String getSystemProperty(String key) {
        String ret;
        try {
            @SuppressWarnings("rawtypes")
            Class SystemProperties = Class.forName("android.os.SystemProperties");

            @SuppressWarnings("rawtypes")
            Class[] paramTypes = new Class[1];
            paramTypes[0] = String.class;

            @SuppressWarnings("unchecked")
            Method get = SystemProperties.getMethod("get", paramTypes);

            Object[] params = new Object[1];
            params[0] = key;

            ret = (String) get.invoke(SystemProperties, params);

        } catch (Exception e) {
            Log.e(Constants.TAG, e.getMessage(), e);
            ret = "";
        }

        return ret;
    }
}
