package org.wfp.mpos.utils.sn;

import android.util.Log;

/**
 * Created by othomas on 06/01/2018.
 */

public class BatchDef {


    private static final String TAG = "BatchDef";

    // see https://www.gs1.org/docs/barcodes/GS1_General_Specifications.pdf
    private static final int LEN_GTIN = 14;
    // trade item application identifier + GTIN-14 + S/N application identifier + S/N
    //public static final String FX_BARCODE_PATTERN = "0103770004396(\\d{3})21([0-9A-Z]{1,20})"
    public static final String FX_BARCODE_PATTERN = String.format("01(\\d{%d})21([0-9A-Z]{1,20})", LEN_GTIN);
    public static final int BARCODE_PREFIX_LEN = LEN_GTIN + 4;

    //// barcode definitions
    public static final String BARCODE_PREFIX_FX = "0103770004396";

    // IMEI definitions
    // Type Allocation Code
    public static final String IMEI_TAC_FX100 = "35270106";

    public static final String[] IMEI_TAC_FX_ARRAY = new String[]{IMEI_TAC_FX100};

    // MAC definitions
    // see http://standards.ieee.org/develop/regauth/oui/oui.txt
    public static final String OUI = "000CE7"; // MediaTek Inc.

    public static final int WIFI_MAC_BT_ADDR_DIFF = 0x800000;

    private String productName;
    private int batch;
    private int numOfDevices;

    private String barcodePrefix;
    private String firstSN;

    private String imeiTac;
    private int numSimSlots;
    private int firstImeiSN;

    private String firstWifiMacAddr;
    private String firstBtAddr;

    private int firstSNBase10;
    private int lastSNBase10;


    public BatchDef(String product, int batch, String barcodePrefix, int numDevices,
                    String firstSN, String imeiTac, int numSimSlots, int firstImeiSN, String firstWifiMac, String firstBtAddr) {
        this.productName = product;
        this.batch = batch;
        this.barcodePrefix = unformatBarcode(barcodePrefix);
        this.numOfDevices = numDevices;
        this.firstSN = firstSN;
        this.imeiTac = imeiTac;
        this.numSimSlots = numSimSlots;
        this.firstImeiSN = firstImeiSN;
        this.firstWifiMacAddr = firstWifiMac;
        this.firstBtAddr = firstBtAddr;

        firstSNBase10 = SNUtils.getSNBase10(firstSN);
        lastSNBase10 = firstSNBase10 + numDevices - 1;
    }

    public String getProductName() {
        return productName;
    }

    public int getBatch() {
        return batch;
    }

    public int getNumOfDevices() {
        return numOfDevices;
    }

    public String getBarcodePrefix() {
        return barcodePrefix;
    }

    public String getFirstSN() {
        return firstSN;
    }

    public String getImeiTac() {
        return imeiTac;
    }

    public int getFirstImeiSN() {
        return firstImeiSN;
    }

    public int getNumSIMSlots() {
        return numSimSlots;
    }

    public String getFirstWifiMacAddr() {
        return firstWifiMacAddr;
    }


    public String getFirstBtAddr() {
        return firstBtAddr;
    }

    public boolean barcodeInThisModel(String barcode) {
        barcode = unformatBarcode(barcode);

        if (!barcode.startsWith(barcodePrefix))
            return false;

        String snBase36 = barcode.replace(barcodePrefix, "");

        int SN = SNUtils.getSNBase10(snBase36);
        return SN >= firstSNBase10 && SN <= lastSNBase10;
    }

    public boolean imeiInThisModel(String imei) {
        // remove check digit
        imei = imei.substring(0, imei.length() - 1);

        // remove prefix
        imei = imei.replace(imeiTac, "");

        int i = Integer.parseInt(imei);
        return i >= firstImeiSN && i < firstImeiSN + numOfDevices * numSimSlots;
    }

    public static String unformatBarcode(String barcode) {
        barcode = barcode.replaceAll("[() ]", "");
        return barcode;
    }

    public static String formatBarcode(String barcode) {
        try {
            String b1 = barcode.substring(0, 2);
            String b2 = barcode.substring(2, 16);
            String b3 = barcode.substring(16, 18);
            String b4 = barcode.substring(18);
            return String.format("(%s)%s(%s)%s", b1, b2, b3, b4);
        } catch (Exception e) {
            Log.e(TAG, e.getMessage(), e);
            return "";
        }
    }

    public static String[] splitBarcode(String barcode) {
        barcode = unformatBarcode(barcode);
        return new String[]{barcode.substring(0, BARCODE_PREFIX_LEN), barcode.substring(BARCODE_PREFIX_LEN)};
    }
}