package org.wfp.mpos.utils.sn;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by othomas on 06/01/2018.
 */

public class Device {

    private String productName;

    private int batch;

    private int SN;

    private String barcode;

    private List<String> imeiList;

    private String wifiMacAddr;

    private String btAddr;


    private static int getBatchDefIndexFromBarcode(String barcode) {
        for (int i = 0; i < BatchDefTable.DEVICE_BATCH_DEFS.length; i++) {
            BatchDef def = BatchDefTable.DEVICE_BATCH_DEFS[i];
            if (def != null && def.barcodeInThisModel(barcode))
                return i;
        }

        return -1;
    }

    private static int getBatchDefIndexFromImei(String imei) {
        for (int i = 0; i < BatchDefTable.DEVICE_BATCH_DEFS.length; i++) {
            BatchDef def = BatchDefTable.DEVICE_BATCH_DEFS[i];
            if (def != null && def.imeiInThisModel(imei))
                return i;
        }

        return -1;
    }

    static Device fromBarcode(String barcode) throws IllegalArgumentException {
        int batch = getBatchDefIndexFromBarcode(barcode);
        if (batch < 0)
            return null;

        String[] split = BatchDef.splitBarcode(barcode);
        int SN = SNUtils.getSNBase10(split[1]);

        return new Device(batch, SN);
    }

    static Device fromIMEI(String imei) {
        int index = getBatchDefIndexFromImei(imei);
        if (index < 0)
            return null;

        BatchDef def = BatchDefTable.DEVICE_BATCH_DEFS[index];
        // remove check digit
        imei = imei.substring(0, imei.length() - 1);

        // remove prefix
        imei = imei.replace(def.getImeiTac(), "");
        int SN = (Integer.parseInt(imei) - def.getFirstImeiSN()) / def.getNumSIMSlots() + Integer.parseInt(def.getFirstSN(), 36);

        return new Device(index, SN);
    }

    public Device(int index, int SN) {
        BatchDef def = BatchDefTable.DEVICE_BATCH_DEFS[index];

        productName = def.getProductName();

        barcode = def.getBarcodePrefix() + Integer.toString(SN, 36).toUpperCase();
        int offset = SN - Integer.parseInt(def.getFirstSN(), 36);

        imeiList = new ArrayList<String>();
        for (int i = 0; i < def.getNumSIMSlots(); i++) {
            String imei = def.getImeiTac() + String.format("%06d", def.getFirstImeiSN() + offset * def.getNumSIMSlots() + i);
            int sum = Luhn.checkSum(imei);
            imei = imei + sum;
            imeiList.add(imei);
        }

        int wifiMacOffset = Integer.parseInt(def.getFirstWifiMacAddr().replace(":", ""), 16);
        int btAddrOffset = Integer.parseInt(def.getFirstBtAddr().replace(":", ""), 16);

        wifiMacAddr = String.format("%s%06X", BatchDef.OUI, (offset + wifiMacOffset) & 0xFFFFFF);
        btAddr = String.format("%s%06X", BatchDef.OUI, (offset + btAddrOffset) & 0xFFFFFF);
    }

    public String getProductName() {
        return productName;
    }

    public int getBatch() {
        return batch;
    }

    public int getSN() {
        return SN;
    }

    public String getBarcode() {
        return barcode;
    }

    public String getReadableBarcode() {
        return BatchDef.formatBarcode(barcode);
    }

    public String getImei() {
        if (imeiList == null)
            return null;
        return imeiList.get(0);
    }

    public List<String> getImeiList() {
        return imeiList;
    }

    public String getWifiMacAddr() {
        return wifiMacAddr;
    }

    public String getBtAddr() {
        return btAddr;
    }
}