package org.wfp.mpos.utils.sn;

/**
 * Created by othomas on 06/01/2018.
 */

public class Luhn {

    /**
     * Validate a number string using Luhn algorithm
     *
     * @param numberString
     * @return
     */
    public static boolean validate(String numberString) {
        int checkDigit = 0;
        if (numberString.length() % 2 != 0) {
            checkDigit = Integer.parseInt(numberString.substring(numberString.length() - 1));
            numberString = numberString.substring(0, numberString.length() - 1);
        }

        return checkSum(numberString) == checkDigit;
    }

    /**
     * Generate check digit for a number string.
     *
     * @param numberString
     * @return
     */
    public static int checkSum(String numberString) {
        if (numberString.length() % 2 != 0) {
            numberString = numberString.substring(0, numberString.length() - 1);
        }

        int sum = 0;

        boolean flipflop = true;
        for (int i = numberString.length(); i > 0; i--) {
            int digit = Integer.parseInt(String.valueOf(numberString.charAt(i-1)));

            if (flipflop)
                sum += sumToSingleDigit(digit * 2);
            else
                sum += digit;

            flipflop = !flipflop;
        }

        int checkDigit = sum % 10;
        if (checkDigit == 0)
            return checkDigit;
        else
            return 10 - checkDigit;
    }

    private static int sumToSingleDigit(int k) {
        return (k / 10) + (k % 10);
    }
}