package org.wfp.mpos.utils.sn;


import android.bluetooth.BluetoothClass;

import java.util.List;

/**
 * Created by othomas on 06/01/2018.
 */

public class SNUtils {

    public static boolean isIMEIValid(String imei) {
        return imei != null && imei.length() == 15 && Luhn.validate(imei);
    }

    public static boolean isFamocoImei(String imei) {
        if (!isIMEIValid(imei))
            return false;

        for (String tac : BatchDef.IMEI_TAC_FX_ARRAY) {
            if (imei.startsWith(tac))
                return true;
        }

        return false;
    }

    public static boolean isFamocoIdValid(String barcode) {
        if (barcode == null) {
            return false;
        }

        barcode = BatchDef.unformatBarcode(barcode);
        return barcode.matches(BatchDef.FX_BARCODE_PATTERN);
    }

    public static String imeiToBarcode(String imei) {
        return imeiToBarcode(imei, false);
    }

    public static String imeiToBarcode(String imei, boolean formatted) {
        if (imei == null)
            return null;
        Device device = Device.fromIMEI(imei);
        if (device == null)
            return null;

        if (formatted)
            return device.getReadableBarcode();
        return device.getBarcode();
    }

    public static List<String> barcodeToImeiList(String barcode) {
        if (barcode == null)
            return null;

        Device device = Device.fromBarcode(barcode);
        if (device == null)
            return null;

        return device.getImeiList();
    }

    public static String barcodeToImei(String barcode) {
        if (barcode == null)
            return null;
        Device device = Device.fromBarcode(barcode);
        if (device == null)
            return null;

        return device.getImei();
    }

    public static String barcodeToWifiMacAddr(String barcode) {
        if (barcode == null)
            return null;
        return barcodeToWifiMacAddr(barcode, false);
    }

    public static String barcodeToWifiMacAddr(String barcode, boolean formatted) {
        if (barcode == null)
            return null;

        Device device = Device.fromBarcode(barcode);
        if (device == null)
            return null;

        String addr = device.getWifiMacAddr();
        if (formatted)
            addr = formatAddr(addr);

        return addr;
    }

    public static String barcodeToBtAddr(String barcode) {
        if (barcode == null)
            return null;

        return barcodeToBtAddr(barcode, false);
    }

    public static String barcodeToBtAddr(String barcode, boolean formatted) {
        if (barcode == null)
            return null;

        Device device = Device.fromBarcode(barcode);
        if (device == null)
            return null;

        String addr = device.getBtAddr();
        if (formatted)
            addr = formatAddr(addr);

        return addr;
    }

    public static String formatBarcode(String barcode) {
        if (barcode == null)
            return null;

        return BatchDef.formatBarcode(barcode);
    }

    public static int getBatchNumberFromBarcode(String barcode) {
        if (barcode == null)
            return -1;

        barcode = BatchDef.unformatBarcode(barcode);

        if (!isFamocoIdValid(barcode))
            return -1;

        for (BatchDef model : BatchDefTable.DEVICE_BATCH_DEFS) {
            if (model != null && model.barcodeInThisModel(barcode))
                return model.getBatch();
        }

        return -1;
    }

    private static String getProductNameFromDevice(Device device) {
        if (device != null && device.getProductName() != null)
            return device.getProductName();

        return "Unknown";
    }

    public static String getProductNameFromBarcode(String barcode) {
        Device device = Device.fromBarcode(barcode);
        return getProductNameFromDevice(device);
    }

    public static String getProductNameFromImei(String imei) {
        Device device = Device.fromIMEI(imei);
        return getProductNameFromDevice(device);
    }

    public static String formatAddr(String addr) {
        StringBuffer sb = new StringBuffer();
        int i = 0;
        for (; i < 5; i++) {
            sb.append(addr.substring(i * 2, i * 2 + 2) + ":");
        }
        sb.append(addr.substring(i * 2, i * 2 + 2));

        return sb.toString();
    }

    public static int getSNBase10(String SN) {
        try {
            return Integer.parseInt(SN, 36);
        } catch (NumberFormatException e) {
            return -1;
        }
    }
}